
#include "mbed.h"
#include "EthernetInterface.h"
#include "SNTPClient.h"
#include "DHT.h"
#include "MQTTEthernet.h"
#include "MQTTClient.h"

//#define ENABLE_ETH
#define ENABLE_NTP
#define ENABLE_DHT
//#define ENABLE_TCPC
#define ENABLE_MQTT

#define INTERVAL_TIME           1.0
#define INTERVAL_PROC			1.0

Ticker flipper;
EthernetInterface eth;
tstamp g_seconds=0;
char dip[4];
int port;
char cfg_tmp[20];
tstamp g_post_seconds=0;
int arrivedcount = 0;

#ifdef ENABLE_DHT
DHT sensor(D4, DHT11);
#endif

void flip() {
    g_seconds += 1;
}

void messageArrived(MQTT::MessageData& md)
{
    MQTT::Message &message = md.message;
    printf("Message arrived: qos %d, retained %d, dup %d, packetid %d\r\n", message.qos, message.retained, message.dup, message.id);
    printf("Payload %.*s\r\n", message.payloadlen, (char*)message.payload);
    ++arrivedcount;
}

void baud(int baudrate) {
    Serial s(USBTX, USBRX);
    s.baud(baudrate);
}

int main (void)
{
    datetime time;
    int ret;
#ifdef ENABLE_ETH
    uint8_t mac_addr[6] = {0x00, 0x08, 0xDC, 0x01, 0x02, 0x03};
#endif
#ifdef ENABLE_DHT
    int error = 0;
    float h = 0.0f, c = 0.0f;
#endif
#ifdef ENABLE_ETH
	char client_str[100];
#endif
	
    baud(115200);

    printf("#Wait a second...\r\n");
#ifdef ENABLE_ETH
    eth.init(mac_addr); //Use DHCP
    eth.connect();
    printf("IP Address is %s\r\n", eth.getIPAddress());
#endif

#ifdef ENABLE_ETH
	dip[0] = 192;
	dip[1] = 168;
	dip[2] = 0;
	dip[3] = 2;
	port = 5000;
#endif

#ifdef ENABLE_MQTT
	MQTTEthernet ipstack = MQTTEthernet();
	MQTT::Client<MQTTEthernet, Countdown> client = MQTT::Client<MQTTEthernet, Countdown>(ipstack);

	char* hostname = "dmqtt.thingplus.net";
	int port = 1883;
	
	int rc = ipstack.connect(hostname, port);
	if (rc != 0)
		printf("rc from TCP connect is %d\r\n", rc);
		
	char MQTTClientID[30];
	
	MQTTPacket_connectData data = MQTTPacket_connectData_initializer;		
	data.MQTTVersion = 3;
	sprintf(MQTTClientID,"0008dc010203");
	data.clientID.cstring = MQTTClientID;
	data.username.cstring = "0008dc010203";
	data.password.cstring = "MF8_wXPMLY10FHF5b_VXmBjhWxxx";  

	if ((rc = client.connect(data)) != 0)
		printf("rc from MQTT connect is %d\r\n", rc);
		
	MQTT::Message message;
	char buf[100];
	//int error = 0;
#endif	  

    
#ifdef ENABLE_NTP
    SNTPClient sntp("time.nist.gov", 40);   // timezone: Korea, Republic of
    //SNTPClient sntp("kr.pool.ntp.org", 40);   // timezone: Korea, Republic of
    sntp.connect();

    do {
        ret = sntp.getTime(&time);
        wait(0.1);
    }while(ret != true);
    printf("1st %d/%d/%d %d:%d:%d \r\n", time.yy, time.mo, time.dd, time.hh, time.mm, time.ss);
#endif   

#ifdef ENABLE_TCPC
	sprintf(cfg_tmp,"%d.%d.%d.%d", dip[0], dip[1], dip[2], dip[3]);
		
	TCPSocketConnection sock;
	//sock.connect("192.168.0.2", 5000);
	sock.connect(cfg_tmp, port);
#endif

	flipper.attach(&flip, INTERVAL_TIME);
	g_post_seconds = g_seconds;

    while (true)
    {
#ifdef ENABLE_NTP
        sntp.calcdatetime(g_seconds);
        sntp.getTime2(&time);
        
        printf("%d/%d/%d %d:%d:%d\r\n", time.yy, time.mo, time.dd, time.hh, time.mm, time.ss);
#endif        

#ifdef ENABLE_DHT
        error = sensor.readData();
        if (0 == error) {
			c	= sensor.ReadTemperature();
			h	= sensor.ReadHumidity();
			printf("Temperature in Celcius: %4.2f\r\n", c);
			printf("Humidity is %4.2f\r\n", h);
        } else {
            printf("Error: %d\r\n", error);
        }
#endif

#ifdef ENABLE_TCPC
		memset(client_str, 0x00, 100);
#ifdef ENABLE_NTP
		sprintf(client_str, "%d/%d/%d %d:%d:%d, %4.2f, %4.2f\r\n", time.yy, time.mo, time.dd, time.hh, time.mm, time.ss, c, h);
#else
		sprintf(client_str, "%4.2f, %4.2f\r\n", c, h);
#endif		  
		sock.send_all(client_str, sizeof(client_str));
#endif

#ifdef ENABLE_MQTT
		message.qos = MQTT::QOS0;
		message.retained = false;
		message.dup = false;
				
		if((g_seconds-g_post_seconds) > 60)
		{
			sprintf(buf, "%lu000,%.2f", g_seconds, c);
			message.payload = (void*)buf;
			message.payloadlen = strlen(buf);
			rc = client.publish("v/a/g/0008dc010203/s/temperature-0008dc010203-0", message);
		
			g_post_seconds = g_seconds;  
		}
#endif		  

		wait(INTERVAL_PROC);
	
    }
}
